@@ -2,43 +2,42 @@
 IPRoute module
 ==============
 
-`iproute` module provides low-level API to RTNetlink protocol
-via `IPRoute` and `IPRSocket` classes as well as all required
-constants.
-
 iproute quickstart
 ------------------
 
-**pyroute2.IPRoute** in two words::
+**IPRoute** in two words::
 
     $ sudo pip install pyroute2
-    $ cat >example.py <<EOF
-    > from pyroute2 import IPRoute
-    > ip = IPRoute()
-    > print([x.get_attr('IFLA_IFNAME') for x in ip.get_links()])
-    > EOF
+
+    $ cat example.py
+    from pyroute2 import IPRoute
+    ip = IPRoute()
+    print([x.get_attr('IFLA_IFNAME') for x in ip.get_links()])
+
     $ python example.py
     ['lo', 'p6p1', 'wlan0', 'virbr0', 'virbr0-nic']
 
 threaded vs. threadless architecture
 ------------------------------------
 
-Please note, that objects of `IPRoute` class implicitly starts
-several threads:
+Since v0.3.2, IPRoute class is threadless by default.
+It spawns no additional threads, and receives only
+responses to own requests, no broadcast messages. So,
+if you prefer not to cope with implicit threading, you
+can safely use this module.
 
-* I/O Loop -- main thread that performs all Netlink I/O and clusterization
-* Main thread -- thread that reassembles messages and parses them into
-  dict-like structures
-* Cache thread -- `IPRoute` objects can be connected together, and in
-  this case header masquerading should be performed on netlink packets;
-  the thread performs masquerade cache expiration
+To get broadcast messages, use `IPRoute.bind()` call.
+Please notice, that after calling `IPRoute.bind()` you
+MUST get all the messages in time. In the case of the
+kernel buffer overflow, you will have to restart the
+socket.
 
-In most cases it should be ok, `IPRoute` uses no daemonic threads and
-explicit `release()` call is provided to stop all the threads. Beside
-of that, the architecture provides packet buffering.
-
-But if you do not like implicit threads, you can use simplest
-threadless RTNetlink interface, `IPRSocket`.
+With `IPRoute.bind(async=True)` one can launch async
+message receiver thread with `Queue`-based buffer. The
+buffer is thread-safe and completely transparent from
+the programmer's perspective. Please read also
+`NetlinkSocket` documentation to know more about async
+mode.
 
 classes
 -------
@@ -57,9 +56,6 @@ from pyroute2.netlink import NLM_F_ACK
 from pyroute2.netlink import NLM_F_DUMP
 from pyroute2.netlink import NLM_F_CREATE
 from pyroute2.netlink import NLM_F_EXCL
-from pyroute2.netlink import NETLINK_ROUTE
-from pyroute2.netlink.client import Netlink
-from pyroute2.netlink.rtnl import RTNL_GROUPS
 from pyroute2.netlink.rtnl import RTM_NEWADDR
 from pyroute2.netlink.rtnl import RTM_GETADDR
 from pyroute2.netlink.rtnl import RTM_DELADDR
@@ -92,7 +88,7 @@ from pyroute2.netlink.rtnl import TC_H_ROOT
 from pyroute2.netlink.rtnl import rtprotos
 from pyroute2.netlink.rtnl import rtypes
 from pyroute2.netlink.rtnl import rtscopes
-from pyroute2.netlink.rtnl import MarshalRtnl
+from pyroute2.netlink.rtnl.req import IPLinkRequest
 from pyroute2.netlink.rtnl.tcmsg import get_htb_parameters
 from pyroute2.netlink.rtnl.tcmsg import get_htb_class_parameters
 from pyroute2.netlink.rtnl.tcmsg import get_tbf_parameters
@@ -107,6 +103,7 @@ from pyroute2.netlink.rtnl.brmsg import brmsg
 from pyroute2.netlink.rtnl.bomsg import bomsg
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
+from pyroute2.netlink.rtnl import IPRSocket
 
 from pyroute2.common import basestring
 
@@ -120,7 +117,7 @@ def transform_handle(handle):
     return handle
 
 
-class IPRoute(Netlink):
+class IPRoute(IPRSocket):
     '''
     You can think of this class in some way as of plain old iproute2
     utility.
@@ -144,89 +141,7 @@ class IPRoute(Netlink):
             # bring it up
         ipr.link('set', dev, state='up')
 
-    *Usage*
-
-    IPRoute objects allows not only simple monitoring or querying
-    of RT netlink, but also clusterization of IPRoute instances.
-    Simple local sample:
-
-        >>> from pyroute2 import IPRoute
-        >>> from pprint import pprint
-        >>> ip = IPRoute()
-        >>> ip.monitor()
-        >>> pprint(ip.get())
-        [{'attrs': [('RTA_TABLE', 255),
-                    ('RTA_DST', 'ff02::1:2'),
-                    ('RTA_OIF', 3),
-                    ('RTA_PRIORITY', 0),
-                    ('RTA_CACHEINFO', {'rta_clntref': 1,
-                                       'rta_error': 0,
-                                       'rta_expires': 0,
-                                       'rta_id': 0,
-                                       'rta_lastuse': 0,
-                                       'rta_ts': 0,
-                                       'rta_tsage': 0,
-                                       'rta_used': 0})],
-          'dst_len': 128,
-          'event': 'RTM_NEWROUTE',
-          'family': 10,
-          'flags': 512,
-          'header': {'error': None,
-                     'flags': 0,
-                     'host': 'netlink://16',
-                     'length': 108,
-                     'pid': 0,
-                     'sequence_number': 0,
-                     'type': 24},
-          'proto': 0,
-          'scope': 0,
-          'src_len': 0,
-          'table': 255,
-          'tos': 0,
-          'type': 1}]
-        >>>
-
-    IPRoute objects have many methods to get the information
-    about Linux network objects:
-
-        >>> pprint(ip.get_routes()[0])
-        {'attrs': [('RTA_TABLE', 254),
-                   ('RTA_GATEWAY', '10.34.131.254'),
-                   ('RTA_OIF', 2)],
-         'dst_len': 0,
-         'event': 'RTM_NEWROUTE',
-         'family': 2,
-         'flags': 0,
-         'proto': 4,
-         'scope': 0,
-         'src_len': 0,
-         'table': 254,
-         'tos': 0,
-         'type': 1}
-        >>> pprint(ip.get_neighbors()[0])
-        {'attrs': [('NDA_DST', 'ff02::2'),
-                   ('NDA_LLADDR', '33:33:00:00:00:02'),
-                   ('NDA_PROBES', 0),
-                   ('NDA_CACHEINFO', {'ndm_confirmed': 309550224,
-                                      'ndm_refcnt': 0,
-                                      'ndm_updated': 309544224,
-                                      'ndm_used': 309544224})],
-         'event': 'RTM_NEWNEIGH',
-         'family': 10,
-         'flags': 0,
-         'ifindex': 33554432,
-         'ndm_type': 64,
-         'state': 0}
-        >>>
-
-    But IPRoute objects start additional threads to implement
-    transparent authentication, message reassembling and so on.
-    Sometimes it can become an overkill for simple projects, in
-    these cases consider usage of IPRSocket.
     '''
-    marshal = MarshalRtnl
-    family = NETLINK_ROUTE
-    groups = RTNL_GROUPS
 
     # 8<---------------------------------------------------------------
     #
@@ -391,6 +306,20 @@ class IPRoute(Netlink):
                 if (x.get_attr('RTA_DST', None) is None and
                     x['dst_len'] == 0)]
 
+    def link_create(self, **kwarg):
+        '''
+        Create a link. The method parameters will be
+        passed to the `IPLinkRequest()` constructor as
+        a dictionary.
+
+        Examples::
+
+            ip.link_create(ifname='very_dummy', kind='dummy')
+            ip.link_create(ifname='br0', kind='bridge')
+            ip.link_create(ifname='v101', kind='vlan', vlan_id=101, link=1)
+        '''
+        return self.link('add', **IPLinkRequest(kwarg))
+
     def link_up(self, index):
         '''
         Switch an interface up unconditionally.
@@ -451,14 +380,11 @@ class IPRoute(Netlink):
         '''
         flags = NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL | NLM_F_REQUEST
         ret = []
-        debug = self.debug
-        self.debug = True
         kwarg['table'] = kwarg.get('table', DEFAULT_TABLE)
         for route in self.get_routes(*argv, **kwarg):
             ret.append(self.nlm_request(route,
                                         msg_type=RTM_DELROUTE,
                                         msg_flags=flags))
-        self.debug = debug
         return ret
     # 8<---------------------------------------------------------------
 
@@ -503,9 +429,11 @@ class IPRoute(Netlink):
             ip.link("delete", index=x)
         '''
 
-        commands = {'set': RTM_SETLINK,      # almost all operations
-                    'add': RTM_NEWLINK,      # no idea, how to use it :)
-                    'delete': RTM_DELLINK}   # remove interface
+        commands = {'set': RTM_SETLINK,
+                    'add': RTM_NEWLINK,
+                    'del': RTM_DELLINK,
+                    'remove': RTM_DELLINK,
+                    'delete': RTM_DELLINK}
         command = commands.get(command, command)
 
         msg_flags = NLM_F_REQUEST | NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL
@@ -573,16 +501,19 @@ class IPRoute(Netlink):
         '''
 
         commands = {'add': RTM_NEWADDR,
+                    'del': RTM_DELADDR,
+                    'remove': RTM_DELADDR,
                     'delete': RTM_DELADDR}
         command = commands.get(command, command)
 
         flags = NLM_F_REQUEST | NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL
 
         # try to guess family, if it is not forced
-        if family is None and address.find(":") > -1:
-            family = AF_INET6
-        else:
-            family = AF_INET
+        if family is None:
+            if address.find(":") > -1:
+                family = AF_INET6
+            else:
+                family = AF_INET
 
         msg = ifaddrmsg()
         msg['index'] = index
@@ -695,6 +626,7 @@ class IPRoute(Netlink):
 
         commands = {'add': RTM_NEWQDISC,
                     'del': RTM_DELQDISC,
+                    'remove': RTM_DELQDISC,
                     'delete': RTM_DELQDISC,
                     'add-class': RTM_NEWTCLASS,
                     'del-class': RTM_DELTCLASS,
@@ -793,6 +725,8 @@ class IPRoute(Netlink):
         # 8<----------------------------------------------------
         commands = {'add': (RTM_NEWROUTE, flags_make),
                     'set': (RTM_NEWROUTE, flags_replace),
+                    'del': (RTM_DELROUTE, flags_make),
+                    'remove': (RTM_DELROUTE, flags_make),
                     'delete': (RTM_DELROUTE, flags_make)}
         (command, flags) = commands.get(command, command)
         msg = rtmsg()
@@ -890,6 +824,8 @@ class IPRoute(Netlink):
             raise 'unsupported table number'
 
         commands = {'add': RTM_NEWRULE,
+                    'del': RTM_DELRULE,
+                    'remove': RTM_DELRULE,
                     'delete': RTM_DELRULE}
         command = commands.get(command, command)
 
